@@ -12,9 +12,9 @@ use Try::Tiny;
 use Exporter qw(import);
 use POSIX;
 
-our $VERSION = '0.102';    # VERSION
+our $VERSION = '0.103';    # VERSION
 
-our @EXPORT_OK = qw(run reader writer);
+our @EXPORT_OK = qw(run run_cb reader writer);
 
 sub _rpipe {
     my $on_eof = shift;
@@ -65,6 +65,14 @@ sub _on_read_helper {
     );
 }
 
+sub _read_on_scalar {
+    my ( $var, $sub ) = @_;
+    my $old = $$var || '';
+    tie $$var, __PACKAGE__ . '::TiedScalar', $sub;
+    $$var = $old;
+    $var;
+}
+
 sub _reaper {
     my ( $self, $waiters ) = @_;
     my $sub = sub {
@@ -344,7 +352,28 @@ sub writer {
 }
 
 sub run {
-    my ( $bin, @args ) = @_;
+    my $cv = AE::cv;
+    run_cb(
+        @_,
+        sub {
+            $cv->send( \@_ );
+        }
+    );
+    my ( $out, $err, $status ) = @{ $cv->recv };
+    $? = $status << 8;
+    if (wantarray) {
+        return ( $out, $err );
+    }
+    else {
+        carp $err if $err;
+        return $out;
+    }
+}
+
+sub run_cb {
+    my $bin  = shift;
+    my $cb   = pop;
+    my @args = @_;
     my ( $out, $err ) = ( '', '' );
     my $proc = __PACKAGE__->new(
         bin    => $bin,
@@ -353,14 +382,13 @@ sub run {
         errstr => \$err
     );
     $proc->finish;
-    $? = $proc->wait << 8;
-    if (wantarray) {
-        return ( $out, $err );
-    }
-    else {
-        warn $err if $err;
-        return $out;
-    }
+    $proc->wait(
+        sub {
+            my $status = shift;
+            $? = $status << 8;
+            $cb->( $out, $err, $status );
+        }
+    );
 }
 
 sub _on {
@@ -409,6 +437,7 @@ sub kill {
 
 sub fire_and_kill {
     my $self   = shift;
+    my $cb     = ( ref $_[-1] eq 'CODE' ? pop : undef );
     my $time   = pop;
     my $signal = uc( pop || 'TERM' );
     my $w      = AnyEvent->timer(
@@ -419,9 +448,19 @@ sub fire_and_kill {
         }
     );
     $self->fire($signal);
-    my $exit = $self->wait;
-    undef $w;
-    $exit;
+    if ($cb) {
+        return $self->wait(
+            sub {
+                undef $w;
+                $cb->(@_);
+            }
+        );
+    }
+    else {
+        my $exit = $self->wait;
+        undef $w;
+        return $exit;
+    }
 }
 
 sub alive {
@@ -431,11 +470,28 @@ sub alive {
 }
 
 sub wait {
-    my ($self) = @_;
-    my $status = $self->{cv}->recv;
-    waitpid $self->{pid} => 0;
-    $self->end;
-    $status;
+    my ( $self, $cb ) = @_;
+    my $next = sub {
+        my $status = $self->{cv}->recv;
+        waitpid $self->{pid} => 0;
+        $self->end;
+        $status;
+    };
+    if ($cb) {
+        my $old_cb = $self->{cv}->cb || sub { };
+        my $cv = AE::cv;
+        $self->{cv}->cb(
+            sub {
+                $old_cb->(@_);
+                my $status = $next->();
+                $cv->send( $cb->($status) );
+            }
+        );
+        return $cv;
+    }
+    else {
+        return $next->();
+    }
 }
 
 sub finish {
@@ -592,6 +648,15 @@ sub pull {
             };
         }
     }
+    elsif ( ref $peer eq 'SCALAR' ) {
+        return _read_on_scalar(
+            $peer,
+            sub {
+                AE::log debug => "$peer->STORE";
+                $self->write( shift() );
+            }
+        );
+    }
     elsif ( ref $peer eq 'GLOB' ) {
         return $self->pull( AnyEvent::Handle->new( fh => $peer ) );
     }
@@ -805,7 +870,8 @@ AnyEvent::post_detect {
 
 1;
 
-package AnyEvent::Proc::R;
+package    # hidden
+  AnyEvent::Proc::R;
 
 use overload '""' => sub { shift->{fileno} };
 
@@ -856,7 +922,8 @@ sub readline {
 
 1;
 
-package AnyEvent::Proc::W;
+package    # hidden
+  AnyEvent::Proc::W;
 
 use overload '""' => sub { shift->{fileno} };
 
@@ -901,6 +968,27 @@ sub pull { die 'UNIMPLEMENTED' }
 
 1;
 
+package    # hidden
+  AnyEvent::Proc::TiedScalar;
+
+use Tie::Scalar;
+
+our @ISA = ('Tie::Scalar');
+
+sub TIESCALAR {
+    bless pop, shift;
+}
+
+sub FETCH {
+    undef;
+}
+
+sub STORE {
+    shift->(pop);
+}
+
+1;
+
 __END__
 
 =pod
@@ -911,7 +999,7 @@ AnyEvent::Proc - Run external commands
 
 =head1 VERSION
 
-version 0.102
+version 0.103
 
 =head1 SYNOPSIS
 
@@ -1033,13 +1121,13 @@ Kills the subprocess the most brutal way. Equals to
 
 	$proc->fire('kill')
 
-=head2 fire_and_kill([$signal, ]$time)
+=head2 fire_and_kill([$signal, ]$time[, $callback])
 
 Fires specified signal C<$signal> (or I<TERM> if omitted) and after C<$time> seconds kills the subprocess.
 
-This is a synchronous call. After this call, the subprocess can be considered to be dead.
+See L</wait> for the meaning of the callback parameter and return value.
 
-Returns the exit code of the subprocess.
+Without calllback, this is a synchronous call. After this call, the subprocess can be considered to be dead. Returns the exit code of the subprocess.
 
 =head2 alive()
 
@@ -1049,9 +1137,11 @@ In fact, the method equals to
 
 	$proc->fire(0)
 
-=head2 wait()
+=head2 wait([$callback])
 
-Waits for the subprocess to be finished returns the exit code.
+Waits for the subprocess to be finished call the callback with the exit code. Returns a condvar, sended with the result of the callback.
+
+Without callback, this is a synchronous call returning the exit code.
 
 =head2 finish()
 
@@ -1107,7 +1197,7 @@ C<$fd> defaults to I<stdout>.
 
 =head2 pull($peer)
 
-Pulls any data from another handle to STDIN. C<$peer> maybe another L<AnyEvent::Proc> instance, an L<AnyEvent::Handle>, an L<IO::Handle> (including any subclass), a L<Coro::Channel> or a GlobRef.
+Pulls any data from another handle to STDIN. C<$peer> maybe another L<AnyEvent::Proc> instance, an L<AnyEvent::Handle>, an L<IO::Handle> (including any subclass), a L<Coro::Channel>, a ScalarRef or a GlobRef.
 
 	$proc->pull($socket);
 
@@ -1260,6 +1350,15 @@ The exit-code is stored in C<$?>. Please keep in mind that for portability reaso
 
 	$exitcode = $? >> 8
 
+=head2 run_cb($bin[, @args], $callback)
+
+Like L</run>, but asynchronous will callback. Returns the condvar. See L</wait> for more information.
+
+	AnyEvent::Proc::run_cb($bin, @args, sub {
+		my ($out, $err) = @_;
+		...;
+	});
+
 =head1 EXPORTS
 
 Nothing by default. The following functions will be exported on request:
@@ -1268,6 +1367,8 @@ Nothing by default. The following functions will be exported on request:
 
 =item * L</run>
 
+=item * L</run_cb>
+
 =item * L</reader>
 
 =item * L</writer>
